@@ -10,6 +10,8 @@ REPLICATOR_PORT = 2193
 MANAGER_PORT = 2194
 REGISTRY_PORT = 2195
 CONTROLLER_PORT = 2196
+LB_UPDATER_PORT = 80
+
 
 CORE_PORTS = {
     'auth-service': AUTH_PORT,
@@ -17,6 +19,7 @@ CORE_PORTS = {
     'core-manager': MANAGER_PORT,
     'application-registry': REGISTRY_PORT,
     'cluster-controller': CONTROLLER_PORT,
+    'lbupdater': LB_UPDATER_PORT,
 }
 
 CORE_DOMAINS = {
@@ -25,6 +28,7 @@ CORE_DOMAINS = {
     'core-manager': 'manager',
     'application-registry': 'registry',
     'cluster-controller': 'controller',
+    'lbupdater': 'lbupdater',
 }
 
 
@@ -171,52 +175,16 @@ class ClusterService:
         return self._request('PUT', requests.put, path, *args, **kwargs)
 
 
-class CoreService(ClusterService):
-    def __init__(self, endpoints, service_name, credentials, cacert=None):
-        super().__init__(credentials, cacert)
-        self.log = getLogger('pequod.core-service')
-        self._endpoints = set(endpoints.get(service_name, []))
-        self._rotated_endpoints = sorted(self._endpoints)
-        self.master_endpoint = None
-        self.endpoints
-        self.service_name = service_name
-        self.port = CORE_PORTS[service_name]
-
-    @property
-    def endpoints(self):
-        if self.master_endpoint:
-            return [self.master_endpoint]
-        return list(self._endpoints)
+class BaseService(ClusterService):
 
     @property
     def name(self):
         return self.service_name
 
-    def _request(self, method_name, method, path, use_credentials=True, recursion_depth=0, *args, **kwargs):
-        try:
-            return super()._request(method_name, method, path, use_credentials, *args, **kwargs)
-        except requests.HTTPError as e:
-            if e.response.status_code == 451:
-                if recursion_depth > len(self._rotated_endpoints):
-                    raise
-
-                # we are contacting a service which is not the master
-                master = e.response.headers.get('X-Pequod-Master')
-                if not master:
-                    raise
-                url = urllib.parse.urlparse(e.response.url)
-                for ep in self._rotated_endpoints:
-                    ep_url = urllib.parse.urlparse(ep)
-                    if ep_url.hostname != url.hostname:
-                        self.master_endpoint = ep
-                        break
-                self._rotated_endpoints = self._rotated_endpoints[1:] + self._rotated_endpoints[:1]
-
-                return self._request(method_name, method, path, use_credentials,
-                                     recursion_depth=recursion_depth + 1, *args, **kwargs)
-            raise
-
     def status(self):
+        if not self._endpoints:
+            yield (None, False, 'no endpoints')
+            return
         all_endpoints = sorted(self._endpoints)
         try:
             for ep in all_endpoints:
@@ -243,3 +211,56 @@ class CoreService(ClusterService):
             perm['path_comments'] = path_comments
 
         return perms
+
+
+class AppService(BaseService):
+    def __init__(self, endpoints, service_name, credentials, cacert=None):
+        super().__init__(credentials, cacert)
+        self.log = getLogger('pequod.app-service')
+        self._endpoints = set(endpoints.get(service_name, []))
+        self.service_name = service_name
+
+    @property
+    def endpoints(self):
+        return list(self._endpoints)
+
+
+class CoreService(BaseService):
+    def __init__(self, endpoints, service_name, credentials, cacert=None):
+        super().__init__(credentials, cacert)
+        self.log = getLogger('pequod.core-service')
+        self._endpoints = set(endpoints.get(service_name, []))
+        self._rotated_endpoints = sorted(self._endpoints)
+        self.master_endpoint = None
+        self.service_name = service_name
+        self.port = CORE_PORTS[service_name]
+
+    @property
+    def endpoints(self):
+        if self.master_endpoint:
+            return [self.master_endpoint]
+        return list(self._endpoints)
+
+    def _request(self, method_name, method, path, use_credentials=True, recursion_depth=0, *args, **kwargs):
+        try:
+            return super()._request(method_name, method, path, use_credentials, *args, **kwargs)
+        except requests.HTTPError as e:
+            if e.response.status_code == 451:
+                if recursion_depth > len(self._rotated_endpoints):
+                    raise
+
+                # we are contacting a service which is not the master
+                master = e.response.headers.get('X-Pequod-Master')
+                if not master:
+                    raise
+                url = urllib.parse.urlparse(e.response.url)
+                for ep in self._rotated_endpoints:
+                    ep_url = urllib.parse.urlparse(ep)
+                    if ep_url.hostname != url.hostname:
+                        self.master_endpoint = ep
+                        break
+                self._rotated_endpoints = self._rotated_endpoints[1:] + self._rotated_endpoints[:1]
+
+                return self._request(method_name, method, path, use_credentials,
+                                     recursion_depth=recursion_depth + 1, *args, **kwargs)
+            raise
